import "./NameInput.css";
import { isNameValid as getNameValidity } from "../../../mock-api/apis.js";
import { useState, useEffect, useRef } from "react";
import { Subject, from, switchAll } from "rxjs";

// The native api does not include the name that was checked
// This can cause an incorrect determination of the validity
// of the name if the name is changed before the api call is
// completed. This function wraps the api call to include the
// name that was checked to compare with the current name.
function isNameValid(name) {
  return new Promise((resolve) => {
    getNameValidity(name).then((value) => {
      resolve({ name: name, isValid: value });
    });
  });
}

// Checks if the name is valid
// Shares the name with any listeners (anywhere!)
// Finite number of renders -> no pass by props re-rendering
// Guarantees the following
//   A subscriber will always get the latest name
//   A subscriber will always know if the 'settled' name is valid
//   A subscriber can read if a name has not 'settled' and is still pending validation
export default function NameInput() {
  const [name, setName] = useState("");
  const [isError, setIsError] = useState(false);
  const [isPending, setIsPending] = useState(false);
  const [isValid, setIsValid] = useState(false);
  const nameRef = useRef(name);

  // Anytime name is updated
  useEffect(() => {
    if (name.length > 0) {
      // Send the name to the api to check if it is valid (async)
      let observable = from(isNameValid(name));
      latestObservable.next(observable);

      // Update the nameRef to validate that the name has not changed
      // after the above api call was made
      nameRef.current = name;

      // Broadcast the latest name and it's pending status
      selectedName.next({ name: name, isValid: "pending" });
      setIsPending(true);
      setIsError(false);
      setIsValid(false);
    } else {
      selectedName.next({ name: "", isValid: "false" });
      setIsPending(false);
      setIsError(false);
      setIsValid(false);
    }
  }, [name]);

  // Mount the observable listener on component mount
  useEffect(() => {
    // Only read the most recent API response and ignore the rest
    latestObservable.pipe(switchAll()).subscribe((nameObject) => {
      // Check that the name has not changed since the api call was made
      if (nameObject.name === nameRef.current) {
        setIsPending(false);
        // Show error if name is invalid
        nameObject.isValid ? setIsError(false) : setIsError(true);
        nameObject.isValid ? setIsValid(true) : setIsValid(false);

        // Broadcast the latest name and it's validity
        selectedName.next(nameObject);
      }
    });
  }, []);

  return (
    <div className="NameInput">
      <span>
        <label htmlFor="name">Name</label>
        <input
          type="text"
          id="name"
          name="name"
          value={name}
          onChange={(e) => {
            setName(e.target.value);
          }}
        />
      </span>
      <span>
        <span></span> {/* Spacer */}
        {isError ? (
          <span id="Form-error">This name has already been taken</span>
        ) : null}
        {isPending ? <span id="Form-pending">Checking...</span> : null}
        {isValid ? <span id="Form-valid">Available!</span> : null}
      </span>
    </div>
  );
}

const latestObservable = new Subject();
export const selectedName = new Subject();
